
/*  ESTIMATION OF VECTOR AUTOREGRESSIVE MODELS WITH CHANGES
                IN REGIME VIA THE EM-ALGORITHM               */

/* written by:   Anders Warne
       e-mail:   anders.warne@texlips.org                    */

/*
--------------------------------------------------------------
*/

/*  This file contains information about the regime models you
    can study. The most general model these routines allow you to
    estimate is given by

    (1)  y(t) = mu(s(t)) + Phi(s(t))*D(t) + A(1,s(t))*y(t-1) + ... +
                A(p,s(t))*y(t-p) + e(t),      t=1,2,...,T,

    where y(t) is an n by 1 vector of observables, D(t) is a d by
    1 vector of deterministic variables,

    (2)               e(t)|s(t) = N(0,V(s(t))),

    and s(t) is a q by 1 vector with "regimes". In particular
    it is assumed that s(t) takes values in {1,2,...,q} and that the
    probability law determining s(t) is given by a Markov chain. The
    probability that s(t)=i conditional on s(t-1)=j is p(i,j). Let
    Pmat be a q by q matrix containing these probabilities with p(i,j)
    being the element in row i and column j.

    What you need to provide the estimation routines with are the
    number of y(t) variables, whether you want to use deterministic
    variables or not (dv = 1 for YES and dv = 0 for NO below), the
    number of states, lags, initial guesses for all parameters, and
    state whether you wish to a priori restrict certain parameters
    (i.e. whether they should vary with the state or not.) The
    latter issue is (at least partly) handled by your choice of
    model, m.                      */

/*  NUMBER OF VARIABLES, LAGS AND OBSERVATIONS   */

n = 2;
dv = 0;
p = 2;
obs = 433;

/*  NOTE: p=0 is ONLY possible with m = 7, 8 and 9 (see the
          descriptions below). */

/*  INPUTTING DATA    */

/*  The matrix "data" should contain all observations you have on
    y(t). In other words it should be of dimension obs times n.  */

load bbbb[obs+1,n] = data/ymm5995.asc;
bbbb = 1200*ln(bbbb);
data = bbbb[2:(obs+1),.]-bbbb[1:obs,.];

/*  LABELS FOR THE VARIABLES   */

mvars = "income, money";

/*  DEFINING DATA FREQUENCY, STARTING & END PERIOD, AND FIRST OBSERVATION */

/*  The variable "freq" can take on values 1 for yearly observa-
    tions, 4 for quarterly, and 12 for monthly. For weekly and
    daily data, make sure that the number of weeks and days,
    respectively are the same for each year. Otherwise, the
    graphics will be slightly inaccurate. For non-annuals data the
    starting period, "sp", should be given in fractional form. For
    example, for quarterly data, the second quarter 1953 should be written
    as    sp = 1953.375  or  53.375. Similarly, the fourth quarter
    should be written:   sp = 1953.875  or 53.875. In other words, the first
    quarter is given by 1/8, the second by 3/8, the third by 5/8, and the
    fourth by 7/8. The end period, "ep", and the first observation, "fobs",
    are expressed similarly.

    NOTICE: sp must always be greater than or equal to fobs. The sample
            length is determined from ep and sp. Whenever sp>fobs the
            observations between sp and fobs won't be used in the
            MA-VAR analysis. Similarly, observations after ep will also
            be excluded.
*/

freq = 12;
sp = 1959+(3/24);
ep = 1995+(3/24);
fobs = 1959+(3/24);

/*  To create the vector D(t) only a linear deterministic trend
    and seasonal dummies are allowed. Trend = 0 means no
    linear deterministic trend, while Trend = 1 gives you such
    a deterministic variable. Similarly, you can use seasonal
    dummies by setting Sea equal to 0 for no seasonals, 4 for
    quarterly centered dummies, and 12 for monthly centered
    dummies. No other seasonals are possible.   */

Trend = 0;
Sea = 0;

/*  NOTE: If you intend to select models 1, 2, 7 or 8 inclusion
          of deterministic trends in D(t) implies that y(t)
          cannot be 2nd order stationary and, thus, inference
          may not be valid. The reason why 2nd order statio-
          narity fails to hold is that the coefficient on
          the deterministic trend is a random variable which
          feeds into the variance of y(t). In fact, it will
          imply that there is a quadratic trend in the variance
          of y(t).                */

/*  Note that if dv = 0, it will not matter what you say about
    D(t) in this section.   */

/*
---------------------------------------------------------------
*/

choosea = zeros(n,1);

/*  PREREMOVAL OF SEASONS    */

preremov = 0;
jointsea = 0;

/*  NOTE: Set preremov=1 if you want to filter out seasonals via
          dummies prior to the estimation of the MS-VAR (zero
          otherwise. By setting jointsea=1 this is achieved jointly,
          and by setting jointsea=0 this is handled variable by
          variable. In the latter case, you need to specify for
          which variables you'd like to filter out seasons. The
          default is for all variables, i.e. the vector choosea
          is set to a unit vector. By setting individual elements
          to zero you can de-select the corresponding variables,
          e.g. choosea = {1, 0} means that seasonals will be removed
          from the first variable but not from the second.

*/

/*
---------------------------------------------------------------
*/

/*  NUMBER OF REGIMES AND INDEPENDENT REGIME VARIABLES   */

/*  The number of regimes for s(t) is given by q. It is possible to
    let s(t) represent more than 1 regime variable. This can of course
    be achieved without imposing any restrictions on Pmat or on the
    other parameters, but for most economic data the information
    about regime shifts may be difficult to detect empirically unless
    specific restrictions are imposed on Pmat. This program allows
    you to define qs independent regime variables, each having 2
    states. Notice that independence greatly reduces the number of
    free parameters in Pmat. Specifically, let P(j) be the 2 by 2
    matrix with Markov probabilities for the j:th independent
    regime variable. Then Pmat = P(1) .*. P(2) .*. ... .*. P(qs)
    Now q = 2^qs. Hence while Pmat generally has q(q-1) free
    parameters, it here has 2qs free parameters. For example,
    with qs = 2 we go from a situation with 12 free parameters
    to one with 4 parameters. Similarly, with qs = 3 we go from
    56 free parameters to only 6. The only test statistic which
    seems likely to be of use in practise is the LM test as it
    does not require estimation under the alternative (if you try
    estimation under the alternative you will typically find that
    the program runs into a corner). This statistic is computed
    by the program and uses the negative of the Hessian matrix to
    estimate the information matrix.             */

/*  NOTE:  q = 2^qs iff you choose qs > 1. You must specify both
           parameters.                    */

q = 2;
qs = 1;

/*  SERIALLY UNCORRELATED MARKOV PROCESS    */

/*  If you want to estimate the p(i,j) parameters under the
    linear restrictions that they are equal to the ergodic
    (long run) probabilities, i.e. the Markov process is
    serially uncorrelated, let  sump=1. If not, choose any
    other integer, e.g.  sump=0. (Currently NOT available
    with qs>1).      */

sump = 0;

/*  MODEL DEFINITIONS    */

m = 1;

/*  The integer m can currently take the values 1-9. Below you'll
    find a short description of what these values for m imply.

    m = 1 ;  All coefficients in (1) and (2) vary with s(t)
    m = 2 ;  All coefficients in (1) vary with s(t) while
             V(s(t)) = V  for all s(t).
    m = 3 ;  mu(s(t)) and V(s(t)) vary with s(t), while Phi(s(t)) =
             Phi and A(j,s(t)) = A(j) for all s(t) and j.
    m = 4 ;  Only mu(s(t)) depends on s(t), while all other
             coefficients are regime independent.
    m = 5 ;  Only V(s(t)) depends on s(t), while all other
             coefficients are regime independent.
    m = 6 ;  This model is different from the others. In fact, it
             does not exactly follow the specification in (1).
             Instead, for model 6 we have that:

    (1')   y(t) = mu(s(t)) + Phi*D(t) + A(1)*(y(t-1)-mu(s(t-1))) +...
                + A(p)*(y(t-p)-mu(s(t-p))) + e(t),

             where e(t)|s(t) is N(0,V). Hence, only mu varies with
             the state s, but the VAR is such that it's the deviation
             of y(t-i) from mu(s(t-i)) for i=0,1,...,p that matters.
             This model corresponds to the model examined by Hamilton
             (1994) in chapter 22 of his book "Time Series Analysis".
             As a word of CAUTION, estimation of model 6 is typically
             extremely time consuming, growing exponentially with
             the number of lags.

    m = 7 ;  Same as model 1 except p=0 is possible.
    m = 8 ;  Same as model 2 except p=0 is possible.
    m = 9 ;  Same as model 5 except p=0 is possible. */


/*  INITIAL VALUES FOR THE PARAMETERS    */

/*  To simplify matters (for the computer) we stack all parameters
    in equation (1) into a matrix A of dimension nq by (np+d+1)
    according to

                 mu(1)  Phi(1)  A(1,1) ...  A(p,1)
                 mu(2)  Phi(2)  A(1,2) ...  A(p,2)
    (3)    A =     .      .       .           .
                   .      .       .           .
                 mu(q)  Phi(q)  A(1,q) ...  A(p,q)

    where, e.g., A(1,2) = A(1,s(t)=2). Furthermore, to provide
    the initial values for V(s(t)), they should be stacked
    similarly in a qn by n matrix V, i.e.

                      V(1)
                      V(2)
    (4)           V =   .
                        .
                      V(q)

    while the Markov probabilities are inserted in Pmat. Make
    sure that each column of Pmat sums to unity since the
    probability that s(t) is either 1 or 2 or .. or q given any
    s(t-1) is always 1.           */

A = { 0 .3 0 .2 0,
      12 0 .3 0 .2,
      12 .3 0 .2 0,
      12 0 .3 0 .2 };

V = { 144 0,
      0 144,
      72  0,
      0  72 };

Pmat1 = { .5 .5,
          .5 .5 };
Pmat2 = { .5 .5,
          .5 .5 };

Pmat = Pmat1;

/*  PLEASE make sure that the dimensions of these matrices
    are consistent with your claims about n,dv,p and q. If they
    are not, the computer is bound to inform you eventually. */

/*  LINEAR RESTRICTIONS ON mu(i), Phi(i), A(i,j) AND V(i)    */

/*  If you wish to impose linear restrictions on certain para-
    meters you need to specify the number of restrictions, rest,
    as well as which restrictions you want to impose. The general
    form of the restrictions is given by

    (5)          Rmat*theta - rvec = 0,

    where theta is a column vector containing mu(i), Phi(i),
    A(i,j) and V(i). How they should be ordered depends on your
    choice of model, m. The number of elements of theta is given
    by ds, while Rmat is of dimension rest times ds, and rvec
    is a rest by 1 column vector. What you need to determine is
    Rmat and rvec as well as the number of restrictions, rest.
    You do NOT have to specify theta; the program "knows", based
    on your choice of model, what theta looks like.

    In order to specify Rmat and rvec you need to know how the
    program constructs the vector theta. For each model, m, short
    descriptions are given here:

    m = 1;  vec(A(1)),...,vec(A(q)),vech(V(1)),...,vech(V(q)),
            where the operator vech stacks the unique elements
            of symmetric matrices by taking the elements on and
            below the diagonal. In contrast to the GAUSS command
            vech, this vech reflects column-wise stacking, rather
            than row-wise. Note also that

               A(i) = mu(i) Phi(i) A(1,i) ... A(p,i).

            ds = nq(1+d+np) + qn(n+1)/2

    m = 2;  same as model 1 except that only one of the q V(i)
            matrices is chosen (since they are equal under this
            model).
            ds = nq(1+d+np) + n(n+1)/2

    m = 3;  mu(1),...,mu(q),vec(Phi),vec(A(1)),...,vec(A(p)),
            vech(V(1)),...,vech(V(q)). Notice that only the
            free parameters of A are selected.

            ds = nq + n(d+np) + qn(n+1)/2

    m = 4;  same as model 3 except that only one of the V(i)
            matrices if chosen.

            ds = nq + n(d+np) + n(n+1)/2

    m = 5;  same as model 3 except that only one of the mu(i)
            vectors is chosen.

            ds = n(1+d+np) + qn(n+1)/2

    m = 6;  same as model 4.

    m = 7;  same as model 1.

    m = 8;  same as model 2.

    m = 9;  same as model 5.            */

rest = 0;
rvec = 0;
Rmat = 0;

check = 0;

/*  NOTE: If you do NOT wish to specify restrictions, you must
          set the parameters rest=0, rvec=0 and Rmat=0. If you
          wish the program to check whether you have specified
          the restrictions correctly in terms of the initial
          values for A and V, let the variable check=1; otherwise
          let check=0.       */

/*  MISSPECIFICATION TESTS   */

/*  Whenever you have selected rest=0 the program will compute
    a number of misspecification tests. All tests are based on
    the conditional scores. There are two groups of tests,
    equation-by-equation tests and system tests. The equation-
    by-equation tests only use information from one of the n
    equations at a time, while the system tests takes informa-
    tion from the whole system into account. For each group three
    different statistics are calculated: (i) an autocorrelation
    test; (ii) an ARCH-type test; and (iii) a test investigating
    the Markov chain assumption. The tests are discussed for n=1
    in Hamilton (1991), "Specification Testing in Markov-Switching
    Time-Series Models". The test statistic used is an F-version
    of the conditional score test proposed by White-Newey-Tauchen.
    For the autocorrelation tests the program examines if the
    conditional scores with respect to mu(i) at time tau are
    correlated with the conditional scores with respect to mu(j)
    at time tau-1. For the ARCH tests, the conditional scores at tau
    with respect to the unique elements of V(i) are compared with
    the conditional scores of the unique element of V(j) at time
    tau-1. Finally, for the Markov tests, the conditional scores
    with respect to p(i,i) at tau are compared with the conditional
    scores with respect to p(i,i) and mu(i) at tau-1. In case you
    have selected qs>1, the Markov tests examines if the conditional
    scores with respect to p(i,i;j) at tau are correlated with the
    conditional scores with respect to p(i,i;j) at tau-1. Here,
    p(i,i;j) denotes element (i,i) of P(j). Hence, for qs>1 only
    the system Markov test is computed.

    Degrees of freedom for the tests are calculated as follows.
    Let T = obs-p. Conditional scores are calculated from tau
    equal to 1 to T. One observation is thus "lost" when checking
    if the conditional scores are serially correlated. Let np
    be the number of parameters in one equation. For example, for
    model 1, np = q(np+d+1)+q, while for model 2, np = q(np+d+1)+1.
    Similarly, let ds be the number of parameters in the system
    (see the precise values for ds above) not counting the unique
    Markov probabilities, denoted by qn. For model 1, the number
    of restrictions in the equation-by-equation test for autocorre-
    lation is q^2 while for the system test it's (nq)^2 (the same
    holds for all other models except model 5 where mu(i) does
    not change across states; for the equation-by-equation tests
    the number of restrictions is 1 and for the system test n^2
    for this case). The equation-by-equation test for model 1
    is then compared with an F(q^2,T-1-np-qn) distribution, while the
    system test is compared with an F((nq)^2,T-1-(ds+qn/n)) distribu-
    tion. For those cases when (ds+qn)/n is not an integer, the round-
    ing function in GAUSS is used.

    For the ARCH test, the number of restrictions in the equation-
    by-equation case for model 1 is q^2 (since V(i) varies across
    states), while for model 2 it is 1. In the system tests, the
    number of restrictions for model 1 is (nq(n+1)/2)^2 while it
    is (n(n+1)/2)^2 for model 2. For models 3 and 5 we have the
    same number of restrictions as for model 1, while for models
    4 and 6 we have the same number of restrictions as in model 2.

    For the Markov case, the number of restrictions in the equation-
    by-equation test is 2q (if qs=1) for all models. For the system
    test, the number of restrictions is q(n+1). When you have selec-
    ted qs>1, the system test has 2qs restrictions.      */

/*
------------------------------------------------------------
*/

/*  CONDITIONS FOR ITERATIONS   */

/*  The variables maxit defines the maximum number of iterations
    while convcrit defines the convergence criterion. The convergence
    criterion is applied to a few relevant functions like the maximum
    absolute value of the gradient and the maximum absolute change in
    the estimated parameters.    */

maxit = 500;
convcrit = .1E-8;

/*   OUTPUT FILE   */

output file = results/results.out reset;

/*  Standard errors and tests are based on the Information matrix
    using the conditional scores.     */

/*   TOLERANCE CRITERION FOR NUMERICAL PARTIALS  */

dparm = 1/10000;

/*  This criterion determines the relative change for each parameter
    when calculating numerical partial derivatives (used to compute
    standard errors for (un)conditional means and covariances.  */


